<?php

declare(strict_types=1);

namespace Dedi\SyliusImageOptimizerPlugin\Media\Http\Controller;

use Liip\ImagineBundle\Imagine\Cache\CacheManager;
use Liip\ImagineBundle\Imagine\Data\DataManager;
use Liip\ImagineBundle\Imagine\Filter\FilterManager;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

class ImageResolverOnTheFly implements ImageResolverInterface
{
    private $cacheManager;
    private $dataManager;
    private $filterManager;

    public function __construct(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager
    ) {
        $this->cacheManager = $cacheManager;
        $this->dataManager = $dataManager;
        $this->filterManager = $filterManager;
    }

    public function filter(Request $request, string $filterName): Response
    {
        $queryParamterBag = $request->query;

        $width = $queryParamterBag->get('width');
        $height = $queryParamterBag->get('height');
        $imageName = $queryParamterBag->get('imageName');

        $dirname = pathinfo($imageName, \PATHINFO_DIRNAME);
        $filname = pathinfo($imageName, \PATHINFO_FILENAME);

        $fullfileName = $filname . '_' . $width . '_' . $height;

        $cacheName = $dirname !== '.' ? $dirname . \DIRECTORY_SEPARATOR . $fullfileName : $fullfileName;

        if (!$this->cacheManager->isStored($cacheName, $filterName)) {
            $binary = $this->dataManager->find($filterName, $imageName);

            $filteredBinary = $this->filterManager->applyFilter($binary, $filterName, [
                'filters' => [
                    'thumbnail' => [
                        'size' => [$width, $height],
                        'mode' => 'outbound',
                        'allow_upscale' => false,
                    ],
                ],
            ]);

            $this->cacheManager->store($filteredBinary, $cacheName, $filterName);
        }

        return new RedirectResponse($this->cacheManager->resolve($cacheName, $filterName), Response::HTTP_MOVED_PERMANENTLY);
    }
}
